@@ -2,14 +2,16 @@
 
 __author__ = """rdnfn"""
 __email__ = ""
-__version__ = "0.2.1"
+__version__ = "0.3.0"
 
 import bauwerk.envs.registration
 import bauwerk.utils.logging
+from bauwerk.envs.solar_battery_house import EnvConfig
+from bauwerk.envs.solar_battery_house import SolarBatteryHouseEnv as HouseEnv
 
 try:
     import cvxpy
-    from bauwerk.envs.solvers import solve
+    from bauwerk.envs.solvers import solve  # pylint: disable=ungrouped-imports
 except ImportError:
 
     def solve(env):
@@ -21,5 +23,5 @@ except ImportError:
         )
 
 
-bauwerk.utils.logging.set_log_level("WARNING")
+bauwerk.utils.logging.setup(log_level="WARNING")
 bauwerk.envs.registration.register_all()
